const QLSPrefObserver = 
{
	observe: function(subject, topic, prefName)
	{
		if (topic == "nsPref:changed" && prefName == "general.useragent.locale")
		{
			CheckLocale();
		}
	}
};
const QLSSettingsObserver =
{
	observe: function(subject, topic, state)
	{
		if (topic == "qls-settings" && state == 'OK') ApplySettings();
	}
}
const QLS_ALL_LOCALES = "af-ZA#ast-ES#bg-BG#br-FR#ca-AD#cs-CZ#cy-GB#da-DK#de-DE#el-GR#en-GB#en-US#es-AR#es-ES#eu-ES#fi-FI#fr-FR#ga-IE#gu-IN#he-IL#hu-HU#it-IT#ja-JP#ko-KR#lt-LT#mk-MK#nb-NO#nl-NL#pa-IN#pl-PL#pt-BR#pt-PT#ro-RO#ru-RU#sk-SK#sl-SI#sq-AL#sv-SE#tr-TR#zh-CN#zh-TW";

addEventListener("load",Init,true);

function Init()
{
	this.oPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.");

	if (document.getElementById("mCavemanQLS"))
	{
		var ss = Components.classes[ "@mozilla.org/supports-string;1" ].createInstance(Components.interfaces.nsISupportsString);
		ss.data = document.getElementById("mCavemanQLS").getAttribute('statustext');
		this.oPref.setComplexValue("{25A1388B-6B18-46c3-BEBA-A81915D0DE8F}.description", Components.interfaces.nsISupportsString, ss);
	}

	var oPrefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal);
	oPrefService.addObserver("general.useragent.locale", QLSPrefObserver, false);

	this.oPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("general.useragent.");
	this.oQLSPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.qls.");

	if (!this.oQLSPref.prefHasUserValue("notifyrestart")) this.oQLSPref.setBoolPref("notifyrestart", true);
	if (!this.oQLSPref.prefHasUserValue("askautorestart")) this.oQLSPref.setBoolPref("askautorestart", true);

	if (!this.oQLSPref.prefHasUserValue('customlocale1')) SetQLSPref("customlocale1", "");
	if (!this.oQLSPref.prefHasUserValue('customlabel1')) SetQLSPref("customlabel1", "");
	if (!this.oQLSPref.prefHasUserValue('customlocale2')) SetQLSPref("customlocale2", "");
	if (!this.oQLSPref.prefHasUserValue('customlabel2')) SetQLSPref("customlabel2", "");
	if (!this.oQLSPref.prefHasUserValue('customlocale3')) SetQLSPref("customlocale3", "");
	if (!this.oQLSPref.prefHasUserValue('customlabel3')) SetQLSPref("customlabel3", "");

	if (!this.oQLSPref.prefHasUserValue('visiblemenuitems')) SetQLSPref("visiblemenuitems", QLS_ALL_LOCALES);

	this.oDefPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getDefaultBranch("general.useragent.");
	this.sDefaultLocale = GetDefaultLocale();

	var oObService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService)
	oObService.addObserver(QLSSettingsObserver, "qls-settings", false); 

	ApplySettings();
	InitDefault();
	CheckLocale();
}

function InitDefault()
{
	if (document.getElementById('taskPopup')) InitDefaultSeamonkey();

	var oMItem = document.getElementById("defaultQLS");
	if (oMItem)
	{
		var sLabel = oMItem.getAttribute('statustext') + " (" + this.sDefaultLocale +  ")";

		oMItem.setAttribute("label", sLabel);
		oMItem.setAttribute("ccode", this.sDefaultLocale);
	}
}

function CheckLocale()
{
	var sLocale = GetLocale();
	var oElement = document.getElementById(sLocale);
	if (!oElement) oElement = document.getElementsByAttribute('ccode', sLocale.toLowerCase())[0];
	if (!oElement)
	{
		sLocale = GetSeamonkeyLocale()
		oElement = document.getElementById(sLocale);
		if (!oElement) oElement = document.getElementsByAttribute('ccode', sLocale.toLowerCase())[0];
	}
	if (oElement)
	{
		oElement.setAttribute("hidden", false);
		oElement.setAttribute("checked", true);
	}
	if (document.getElementById("defaultQLS")) document.getElementById("defaultQLS").setAttribute('disabled', sLocale == this.sDefaultLocale);
}

function ChangeLocale(sLocale)
{
	var sCurLocale = GetLocale();
	if (sCurLocale == sLocale) return;

	SetLocale(sLocale);
	CheckLocale();
	
	if (document.getElementById('taskPopup')) ChangeLocaleSeamonkey(sLocale);
	DoLocaleWorkAround();
}

function DoLocaleWorkAround()
{
	//Showing about:config seems to solve the problem of not being able to change non-installed localed (often) in Ff 1.0.7 and 1.4
	window.open("chrome://qls/content/qlsreloadchrome.xul", "qlspleasewait", "chrome,close,titlebar,modal,centerscreen"); 
	var oPromptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
	var checkResult = {};

	try
	{
		//Auto restart in Ff 1.4 and later.
		var oAppStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"].getService(Components.interfaces.nsIAppStartup);
		if (!oAppStartup.eRestart) throw("Restart is not supported in Seamonkey...");
		var bAskRestart = this.oQLSPref.getBoolPref("askautorestart");
		if (bAskRestart == true)
		{
			var sTitle = document.getElementById("mCavemanQLS").getAttribute('label');
			var sMsg = document.getElementById("msCavemanQLS2").getAttribute('statustext');
			var sNotify = document.getElementById("msCavemanQLS").getAttribute('statustext');

			var bRestart = oPromptService.confirmCheck(window,sTitle, sMsg, sNotify, checkResult);

			if (checkResult.value) this.oQLSPref.setBoolPref("askautorestart", false);
		}
		if (bRestart == true || bAskRestart == false) oAppStartup.quit(oAppStartup.eAttemptQuit | oAppStartup.eRestart);
	}
	catch (e)
	{
		var bNotify = this.oQLSPref.getBoolPref("notifyrestart")
	
		if (bNotify == true)
		{
			//Show message in other versions.
			if (document.getElementById("mCavemanQLS") && document.getElementById("mpCavemanQLS"))
			{
				var sTitle = document.getElementById("mCavemanQLS").getAttribute('label');
				var sMsg = document.getElementById("mpCavemanQLS").getAttribute('statustext');
				var sNotify = document.getElementById("msCavemanQLS").getAttribute('statustext');

				oPromptService.alertCheck(window, sTitle, sMsg, sNotify, checkResult);
				if (checkResult.value) this.oQLSPref.setBoolPref("notifyrestart", false);
			}
		}
	}
}

function ChangeLocaleSeamonkey(sLocale)
{
	var QLS_InProfile = true;
	
	var oRDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
	try {var oDS = oRDF.GetDataSourceBlocking("rdf:chrome");}
	catch (e) {return;}

	var inProfile = oDS.GetTarget(oRDF.GetResource("urn:mozilla:package:qls"), oRDF.GetResource("http://www.mozilla.org/rdf/chrome#locType"), true);
	if (inProfile instanceof Components.interfaces.nsIRDFLiteral && inProfile.Value == "install") QLS_InProfile = false;
	
	try
	{
		var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIXULChromeRegistry);
		if (!("selectLocaleForPackage" in chromeRegistry)) chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIChromeRegistrySea);
		chromeRegistry.selectLocaleForPackage(sLocale, "qls", QLS_InProfile);
	}
	catch (e) {return;}
}

function InitDefaultSeamonkey()
{
	if (this.sDefaultLocale.length > 10)
	{
		var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIXULChromeRegistry);
		this.sDefaultLocale = chromeRegistry.getSelectedLocale("global");

		if (("selectLocaleForPackage" in chromeRegistry))
		{
			var QLS_InProfile = true;
			var oRDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
			try {var oDS = oRDF.GetDataSourceBlocking("rdf:chrome");}
			catch (e) {return;}

			var inProfile = oDS.GetTarget(oRDF.GetResource("urn:mozilla:package:mozgest"), oRDF.GetResource("http://www.mozilla.org/rdf/chrome#locType"), true);
			if (inProfile instanceof Components.interfaces.nsIRDFLiteral && inProfile.Value == "install") QLS_InProfile = false;
			var QLSLocale = chromeRegistry.getSelectedLocale("qls");
			chromeRegistry.selectLocaleForPackage(QLSLocale, "qls", QLS_InProfile);	
		}		
	}
}

function OpenSettings()
{
	window.open("chrome://qls/content/qlssettings.xul", "qls-settings", "centerscreen,chrome,width=600,height=305");
}

function ApplySettings()
{
	var oMItem;
	var sLabel = "", sLocale = "";
	var bShowSep = false;
	
	oMItem = document.getElementById("qlscustom1");
	sLocale = GetQLSPref("customlocale1");
	sLabel = GetQLSPref("customlabel1");
	if (sLocale != "" && sLabel != "" && oMItem)
	{
		oMItem.setAttribute("hidden", false);
		oMItem.setAttribute("clocale", sLocale);
		oMItem.setAttribute("ccode", sLocale.toLowerCase());
		oMItem.setAttribute("label", sLabel);
		bShowSep = true;
	}
	oMItem = document.getElementById("qlscustom2");
	sLocale = GetQLSPref("customlocale2");
	sLabel = GetQLSPref("customlabel2");
	if (sLocale != "" && sLabel != "" && oMItem)
	{
		oMItem.setAttribute("hidden", false);
		oMItem.setAttribute("clocale", sLocale);
		oMItem.setAttribute("ccode", sLocale.toLowerCase());
		oMItem.setAttribute("label", sLabel);
		bShowSep = true;
	}
	oMItem = document.getElementById("qlscustom3");
	sLocale = GetQLSPref("customlocale3");
	sLabel = GetQLSPref("customlabel3");
	if (sLocale != "" && sLabel != "" && oMItem)
	{
		oMItem.setAttribute("hidden", false);
		oMItem.setAttribute("clocale", sLocale);
		oMItem.setAttribute("ccode", sLocale.toLowerCase());
		oMItem.setAttribute("label", sLabel);
		bShowSep = true;
	}
	
	if (bShowSep == true) document.getElementById("msCavemanQLS2").setAttribute("hidden", false);
	
	var aAllMItems = QLS_ALL_LOCALES.split("#");
	var aMItems = GetQLSPref("visiblemenuitems").split("#");
	var iIndex, iLen;
	
	iLen = aAllMItems.length;
	for (iIndex = 0; iIndex < iLen; iIndex ++)
	{
		oMItem = document.getElementById(aAllMItems[iIndex]);
		if (oMItem && oMItem.getAttribute("checked") == false) oMItem.setAttribute("hidden", true);
	}
	iLen = aMItems.length;
	for (iIndex = 0; iIndex < iLen; iIndex ++)
	{
		oMItem = document.getElementById(aMItems[iIndex]);
		if (oMItem) document.getElementById(aMItems[iIndex]).setAttribute("hidden", false);
	}
}

function GetLocale()
{
	try {return this.oPref.getComplexValue("locale", Components.interfaces.nsIPrefLocalizedString).data;}
	catch (e) {}
	return this.oPref.getCharPref("locale");
}

function GetDefaultLocale()
{
	try {return this.oDefPref.getComplexValue("locale", Components.interfaces.nsIPrefLocalizedString).data;}
	catch (e) {}
	return this.oDefPref.getCharPref("locale");
}

function GetSeamonkeyLocale()
{
	var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIXULChromeRegistry);
	return chromeRegistry.getSelectedLocale("qls");
}

function SetLocale(sData)
{
	var oPLS = Components.classes["@mozilla.org/pref-localizedstring;1"].createInstance(Components.interfaces.nsIPrefLocalizedString);
	oPLS.data = sData;
	this.oPref.setComplexValue("locale", Components.interfaces.nsIPrefLocalizedString, oPLS);
}

function GetQLSPref(sName)
{
	try {return this.oQLSPref.getComplexValue(sName, Components.interfaces.nsIPrefLocalizedString).data;}
	catch (e) {}
	return this.oQLSPref.getCharPref(sName);
}

function SetQLSPref(sName, sData)
{
	var oPLS = Components.classes["@mozilla.org/pref-localizedstring;1"].createInstance(Components.interfaces.nsIPrefLocalizedString);
	oPLS.data = sData;
	this.oQLSPref.setComplexValue(sName, Components.interfaces.nsIPrefLocalizedString, oPLS);
}
